#include <math.h>

#include "box.h"

#include "debug.h"
#include <iostream>

/* class Xr_AABB */

Xr_AABB::Xr_AABB()
{
}

Xr_AABB::~Xr_AABB()
{
}

void Xr_AABB::setPos(float x, float y, float z)
{
	A_pos.setCoords(x, y, z);
}

void Xr_AABB::genBox(Xr_Mesh *mesh)
{
	int i;
	float Xmin = 0, Xmax = 0, Ymin = 0, Ymax = 0, Zmin = 0, Zmax = 0;
	float *vertex;
	
	if(mesh)
	{
		if(mesh->getVertex())
		{
			vertex = mesh->getVertex();
			for(i = 0; i < mesh->getFaceNumber() * 3 * 3; i += 3)
			{
				if(vertex[i] > Xmax)
				{
					Xmax = vertex[i];
				}
				else if(vertex[i] < Xmin)
				{
					Xmin = vertex[i];
				}
				
				if(vertex[i + 1] > Ymax)
				{
					Ymax = vertex[i + 1];
				}
				else if(vertex[i + 1] < Ymin)
				{
					Ymin = vertex[i + 1];
				}
				
				if(vertex[i + 2] > Zmax)
				{
					Zmax = vertex[i + 2];
				}
				else if(vertex[i + 2] < Zmin)
				{
					Zmin = vertex[i + 2];
				}
			}
			
			A_points[0].setCoords(Xmax, Ymax, Zmax);
			A_points[1].setCoords(Xmax, Ymax, Zmin);
			A_points[2].setCoords(Xmax, Ymin, Zmin);
			A_points[3].setCoords(Xmax, Ymin, Zmax);
			A_points[4].setCoords(Xmin, Ymax, Zmax);
			A_points[5].setCoords(Xmin, Ymax, Zmin);
			A_points[6].setCoords(Xmin, Ymin, Zmin);
			A_points[7].setCoords(Xmin, Ymin, Zmax);
		}
		else
		{
			Xr_Log("Xr_AABB : Le mesh ne contient aucun vertices");
		}
	}
	else
	{
		Xr_Log("Xr_AABB : Le mesh est invalide");
	}
}

Xr_Vector3D Xr_AABB::getPos() const
{
	return A_pos;
}

Xr_Vector3D* const Xr_AABB::getPoints() const
{
	return (Xr_Vector3D* const) A_points;
}


/* class Xr_FrustumBox */

Xr_FrustumBox::Xr_FrustumBox()
{
}

Xr_FrustumBox::~Xr_FrustumBox()
{
}

void Xr_FrustumBox::setBoxSize(float near, float far, float fov, float ratio)
{
	F_far = far;
	F_near = near;

	float radFov = M_PI * (fov / (float) 180);
	
	F_heightFar = 2 * tan(radFov / 2) * F_far;
	F_widthFar = F_heightFar * ratio;
	
	F_heightNear = 2 * tan(radFov / 2) * F_near;
	F_widthNear = F_heightNear * ratio;
}

void Xr_FrustumBox::genBox(Xr_Vector3D const& camPos, Xr_Vector3D const& dir, Xr_Vector3D const& right, Xr_Vector3D const& left, Xr_Vector3D const& up, Xr_Vector3D const& down)
{
	/* 
	 *  box's points
	 *
	 * far plane
	 *
	 *	a +-------------+ b
	 *    |             |
	 *    |             |
	 *    |             |
	 *  c +-------------+ d
     *
     * near plane
     *
	 *	e +-------------+ h
	 *    |             |
	 *    |             |
	 *    |             |
	 *  f +-------------+ g
	 *
	 */

	Xr_Vector3D a, b, c, d, e, f, g, h;

	a = camPos + (dir * F_far) + (up * (F_heightFar / 2)) + (left * (F_widthFar / 2));
	b = camPos + (dir * F_far) + (up * (F_heightFar / 2)) + (right * (F_widthFar / 2));
	c = camPos + (dir * F_far) + (down * (F_heightFar / 2)) + (left * (F_widthFar / 2));
	d = camPos + (dir * F_far) + (down * (F_heightFar / 2)) + (right * (F_widthFar / 2));
	
	e = camPos + (dir * F_near) + (up * (F_heightNear / 2)) + (left * (F_widthNear / 2));
	f = camPos + (dir * F_near) + (down * (F_heightNear / 2)) + (left * (F_widthNear / 2));
	g = camPos + (dir * F_near) + (down * (F_heightNear / 2)) + (right * (F_widthNear / 2));
	h = camPos + (dir * F_near) + (up * (F_heightNear / 2)) + (right * (F_widthNear / 2));
	
	/*std::cout << "v " << a[0] << " " << a[1] << " " << a[2] << std::endl;
	std::cout << "v " << b[0] << " " << b[1] << " " << b[2] << std::endl;
	std::cout << "v " << c[0] << " " << c[1] << " " << c[2] << std::endl;
	std::cout << "v " << d[0] << " " << d[1] << " " << d[2] << std::endl;
	
	std::cout << "v " << e[0] << " " << e[1] << " " << e[2] << std::endl;
	std::cout << "v " << f[0] << " " << f[1] << " " << f[2] << std::endl;
	std::cout << "v " << g[0] << " " << g[1] << " " << g[2] << std::endl;
	std::cout << "v " << h[0] << " " << h[1] << " " << h[2] << std::endl;
	
	std::cout << "f 1 2 3 4\nf 2 8 7 4\nf 1 5 6 3\nf 1 2 8 5\nf 3 4 7 6\n" << std::endl;*/

	F_frustumPlanes[RIGHT].setPoints(g, h, d);
	F_frustumPlanes[LEFT].setPoints(e, f, c);
	F_frustumPlanes[TOP].setPoints(h, e, a);
	F_frustumPlanes[BOTTOM].setPoints(f, g, d);
	F_frustumPlanes[NEAR].setPoints(e, h, g);
}

bool Xr_FrustumBox::testPoint(float x, float y, float z)
{
	Xr_Vector3D point(x, y, z);

	int i;
	
	for(i = 0; i < FRUSTUM_PLANES_NUMBER; i++)
	{
		if(F_frustumPlanes[i].getDistanceToPoint(point) < -MAX_DISTANCE_TEST)
		{
			return false;
		}
	}
	
	return true;
}

bool Xr_FrustumBox::testAABB(Xr_AABB *box)
{
	Xr_Vector3D *points = box->getPoints();
	
	float maxDistance, distance;
	
	int i, j;
	for(i = 0; i < FRUSTUM_PLANES_NUMBER; i++)
	{
		maxDistance = F_frustumPlanes[i].getDistanceToPoint(points[0]);
		
		for(j = 1; j < 8; j++)
		{
			distance = F_frustumPlanes[i].getDistanceToPoint(points[j]);
			
			if(distance > maxDistance)
			{
				maxDistance = distance;
			}
		}
		
		if(maxDistance < -MAX_DISTANCE_TEST)
		{
			return false;
		}
	}
	return true;
}
