#include "light.h"

Xr_Light::Xr_Light() : L_colorRGB({0}), L_intensity(0), L_pos({0}), L_eye({0}), L_fov(0), L_shadow(false), L_projection(new Xr_MATRIX_4X4), L_modelview(new Xr_MATRIX_4X4), L_depthMap(0)
{
	L_projection->loadIdentity();
	
	glGenTextures(1, &L_depthMap);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, L_depthMap);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_INTENSITY);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, DEPTH_MAP_SIZE, DEPTH_MAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, 0);
}

Xr_Light::Xr_Light(float posX, float posY, float posZ, float eyeX, float eyeY, float eyeZ, float intensity, float colorR, float colorG, float colorB, float fov, bool shadow)
 : L_colorRGB({colorR, colorG, colorB}), L_intensity(intensity), L_pos({posX, posY, posZ}), L_eye({eyeX, eyeY, eyeZ}), L_fov(fov), L_shadow(shadow), L_projection(new Xr_MATRIX_4X4), L_modelview(new Xr_MATRIX_4X4), L_depthMap(0)
{
	L_projection->loadIdentity();
	L_modelview->loadIdentity();
	configure(posX, posY, posZ, eyeX, eyeY, eyeZ, intensity, colorR, colorG, colorB, fov, shadow);
	
	glGenTextures(1, &L_depthMap);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, L_depthMap);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_INTENSITY);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, DEPTH_MAP_SIZE, DEPTH_MAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_UNSIGNED_BYTE, 0);
}

Xr_Light::~Xr_Light()
{
	delete L_projection;
	delete L_modelview;
	glDeleteTextures(1, &L_depthMap);
}

void Xr_Light::configure(float posX, float posY, float posZ, float eyeX, float eyeY, float eyeZ, float intensity, float colorR, float colorG, float colorB, float fov, bool shadow)
{
	L_projection->loadProjection(fov, 1, 1, 1000);
	L_modelview->lookAt(eyeX, eyeY, eyeZ, posX, posY, posZ, 0, 1, 0);
	
	L_intensity = intensity;
	
	L_colorRGB[0] = colorR;
	L_colorRGB[1] = colorG;
	L_colorRGB[2] = colorB;
	
	L_shadow = shadow;
}

float* const Xr_Light::getColorRGB() const
{
	return (float* const) L_colorRGB;
}

float Xr_Light::getIntensity() const
{
	return L_intensity;
}

float* const Xr_Light::getPos() const
{
	return (float* const) L_pos;
}

float* const Xr_Light::getEye() const
{
	return (float* const) L_eye;
}

float Xr_Light::getFov() const
{
	return L_fov;
}

Xr_MATRIX_4X4* const Xr_Light::getLightProjectionMatrix() const
{
	return (Xr_MATRIX_4X4* const) L_projection;
}

Xr_MATRIX_4X4* const Xr_Light::getLightModelviewMatrix() const
{
	return (Xr_MATRIX_4X4* const) L_modelview;
}

GLuint Xr_Light::getLightDepthMap() const
{
	return L_depthMap;
}

bool Xr_Light::hasShadow() const
{
	return L_shadow;
}
