#include "material.h"

#include "debug.h"

Xr_Material::Xr_Material() : M_shadow(false), M_shaderless(true), M_ambient(1), M_diffuse(0), M_specular(0), M_texture(0)
{
}

Xr_Material::Xr_Material(bool shadow, bool shaderless, float ambient, float diffuse, float specular, Xr_Texture *texture)
 : M_shadow(shadow), M_shaderless(shaderless), M_ambient(ambient), M_diffuse(diffuse), M_specular(specular)
{
	if(texture)
	{
		if(texture->isLoaded())
		{
			M_texture = texture;
		}
		else
		{
			Xr_Log("Xr_Material : La texture n'est pas chargée");
		}
	}
	else
	{
		Xr_Log("Xr_Material : Aucune texture de sera utilisée");
	}
}

Xr_Material::~Xr_Material()
{
}

void Xr_Material::enableShadow()
{
	M_shadow = true;
}

void Xr_Material::disableShadow()
{
	M_shadow = false;
}

void Xr_Material::enableShaderless()
{
	M_shaderless = true;
}

void Xr_Material::disableShaderless()
{
	M_shaderless = false;
}

void Xr_Material::setAmbient(float ambient)
{
	M_ambient = ambient;
}

void Xr_Material::setDiffuse(float diffuse)
{
	M_diffuse = diffuse;
}

void Xr_Material::setSpecular(float specular)
{
	M_specular = specular;
}

void Xr_Material::attachTexture(Xr_Texture *texture)
{
	if(M_texture == 0)
	{
		if(texture)
		{
			if(texture->isLoaded())
			{
				M_texture = texture;
			}
			else
			{
				Xr_Log("Xr_Material : La texture n'est pas chargée");
			}
		}
		else
		{
			Xr_Log("Xr_Material : La texture n'est pas valide");
		}
	}
	else
	{
		Xr_Log("Xr_Material : Une texture est déjà attachée au materiau");
	}
}

void Xr_Material::detachTexture()
{
	M_texture = 0;
}

float Xr_Material::getAmbient() const
{
	return M_ambient;
}

float Xr_Material::getDiffuse() const
{
	return M_diffuse;
}

float Xr_Material::getSpecular() const
{
	return M_specular;
}

Xr_Texture* const Xr_Material::getTexture() const
{
	return (Xr_Texture* const) M_texture;
}
