#include <fstream>
#include <string>
#include <vector>

#include "mesh.h"

#include "debug.h"


Xr_Mesh::Xr_Mesh() : Xr_Ressource(xr_mesh_type), M_vertex(0), M_texCoord(0), M_normal(0), M_faceNumber(0)
{
}

Xr_Mesh::Xr_Mesh(char *name) : Xr_Ressource(name, xr_mesh_type), M_vertex(0), M_texCoord(0), M_normal(0), M_faceNumber(0)
{
	load();
}

Xr_Mesh::Xr_Mesh(Xr_Mesh const &mesh) : Xr_Ressource(mesh.getName(), xr_mesh_type), M_vertex(0), M_texCoord(0),
M_normal(0),  M_faceNumber(mesh.getFaceNumber())
{
	int i;
	float* const vertex = mesh.getVertex();
	float* const texCoord = mesh.getTexCoord();
	float* const normal = mesh.getNormal();
	
	if(vertex != 0)
	{
		M_vertex = new float[mesh.getFaceNumber() * 3 * 3];
		for(i = 0; i < M_faceNumber * 3 * 3; i++)
		{
			M_vertex[i] = vertex[i];
		}
	}
	
	if(texCoord != 0)
	{
		M_texCoord = new float[mesh.getFaceNumber() * 3 * 2];
		for(i = 0; i < M_faceNumber * 3 * 2; i++)
		{
			M_texCoord[i] = texCoord[i];
		}
	}
	
	if(normal != 0)
	{
		M_normal = new float[mesh.getFaceNumber() * 3 * 3];
		for(i = 0; i < M_faceNumber * 3 * 3; i++)
		{
			M_normal[i] = normal[i];
		}
	}
}

Xr_Mesh::~Xr_Mesh()
{
	delete[] M_vertex;
	delete[] M_texCoord;
	delete[] M_normal;
}

void Xr_Mesh::load()
{
	if(R_name)
	{
		std::ifstream flux(R_name);
		if(flux)
		{
			flux.seekg(0, std::ios::beg);

			std::string word;
			float coord;
			int index, i;
			char c;
			std::vector<float> vertex;
			std::vector<float> texCoord;
			std::vector<float> normal;

			std::vector<int> iVertex;
			std::vector<int> iTexCoord;
			std::vector<int> iNormal;

			// Recherche et récupération des données
			while(!flux.eof())
			{
				flux >> word;

				//Coordonnées de vertice
				if(word == "v")
				{
					flux >> coord; vertex.push_back(coord);
					flux >> coord; vertex.push_back(coord);
					flux >> coord; vertex.push_back(coord);
				}
				//Coordonnées de texture (UV map)
				else if(word == "vt")
				{
					flux >> coord; texCoord.push_back(coord);
					flux >> coord; texCoord.push_back(coord);
				}
				//Coordonnées de normal
				else if(word == "vn")
				{
					flux >> coord; normal.push_back(coord);
					flux >> coord; normal.push_back(coord);
					flux >> coord; normal.push_back(coord);
				}
				//indices
				else if(word == "f")
				{
					M_faceNumber++;
					for(i = 0; i < 3; i++)
					{
						//indice de coordonnées de vertice
						flux >> index; iVertex.push_back(index);
						//flux.seekg(-1, std::ios::cur);

						flux.get(c);
						if(c == ' ')
						{
							continue;
						}

						else if(c == '/')
						{
							flux.get(c);
							if(c == '/') //si on trouve "//" -> pas d'indice de coordonnées de texture
							{
								//indice de coordonnées de normal
								flux >> index; iNormal.push_back(index);
								continue;
							}flux.unget();

							//si on trouve uniquement "/" -> on récupère l'indice de coordonnées de texture
							flux >> index; iTexCoord.push_back(index);
						}

						//si on trouve un espace on passe au indices suivant
						flux.get(c);
						if(c == ' ')
						{
							continue;
						}
						// si on trouve encore "/" on récupère l'indice de coordonnées de normal
						else if(c == '/')
						{
							flux >> index; iNormal.push_back(index);
						}
						//si on est en fin de ligne on relance l'étape de recherche
						else if(c == '\n')
						{
							break;
						}
					}
				}
			}

			//Construction des coordonnées
			if(iVertex.size() != 0)
			{
				M_vertex = new float[iVertex.size() * 3];
				for(i = 0; i < (int) iVertex.size() * 3; i += 3)
				{
					M_vertex[i] = vertex[(iVertex[i / 3] - 1) * 3];
					M_vertex[i + 1] = vertex[(iVertex[i / 3] - 1) * 3 + 1];
					M_vertex[i + 2] = vertex[(iVertex[i / 3] - 1) * 3 + 2];
				}
			}

			if(iTexCoord.size() != 0)
			{
				M_texCoord = new float[iTexCoord.size() * 2];
				for(i = 0; i < (int) iTexCoord.size() * 2; i += 2)
				{
					M_texCoord[i] = texCoord[(iTexCoord[i / 2] - 1) * 2];
					M_texCoord[i + 1] = texCoord[(iTexCoord[i / 2] - 1) * 2 + 1];
				}
			}

			if(iNormal.size() != 0)
			{
				M_normal = new float[iNormal.size() * 3];
				for(i = 0; i < (int) iNormal.size() * 3; i += 3)
				{
					M_normal[i] = normal[(iNormal[i / 3] - 1) * 3];
					M_normal[i + 1] = normal[(iNormal[i / 3] - 1) * 3 + 1];
					M_normal[i + 2] = normal[(iNormal[i / 3] - 1) * 3 + 2];
				}
			}
			R_isLoaded = true;
		}
	}
}

float* const Xr_Mesh::getVertex() const
{
	return (float* const) M_vertex;
}

float* const Xr_Mesh::getTexCoord() const
{
	return (float* const) M_texCoord;
}

float* const Xr_Mesh::getNormal() const
{
	return (float* const) M_normal;
}

int Xr_Mesh::getFaceNumber() const
{
	return M_faceNumber;
}
