#include <string>

#include "object.h"

#include "debug.h"
#include "buffer.h"

Xr_Object::Xr_Object() : O_mesh(0), O_box(0), O_material(0), O_pos({0}), O_isReady(false)
{
}

Xr_Object::~Xr_Object()
{
	if(O_isReady)
	{
		Xr_BufferManager::getInstance()->unload(O_mesh);
	}
	
	if(O_box)
	{
		delete O_box;
	}
}

void Xr_Object::attachMesh(Xr_Mesh *mesh)
{
	if(O_mesh == 0)
	{
		if(mesh)
		{
			if(mesh->isLoaded() != true)
			{
				Xr_Log("Xr_Object : Le mesh n'est pas chargé");
				return;
			}
			O_mesh = mesh;
			
			O_box = new Xr_AABB;
			O_box->genBox(O_mesh);
			O_box->setPos(O_pos[0], O_pos[1], O_pos[2]);
		}
		else
		{
			Xr_Log("Xr_Object : Le mesh est invalide");
		}
	}
	else
	{
		Xr_Log("Xr_Object : Un mesh est déjà attaché à l'objet");
	}
}

void Xr_Object::attachMaterial(Xr_Material *material)
{
	if(O_material == 0)
	{
		if(material)
		{
			O_material = material;
		}
	}
	else
	{
		Xr_Log("Xr_Object : Un materiau est déjà attaché à l'objet");
	}
}

void Xr_Object::detachMesh()
{
	if(O_isReady)
	{
		Xr_BufferManager::getInstance()->unload(O_mesh);
		O_isReady = false;
	}
	delete O_box;
	O_box = 0;
	O_mesh = 0;
}

void Xr_Object::detachMaterial()
{
	O_material = 0;
}

void Xr_Object::loadToServer()
{
	if(Xr_BufferManager::getInstance()->load(O_mesh) && O_box)
	{
		O_isReady = true;
	}
}

void Xr_Object::unloadToServer()
{
	Xr_BufferManager::getInstance()->unload(O_mesh);
	O_isReady = false;
}

void Xr_Object::draw()
{
	if(O_isReady)
	{
		Xr_BufferManager::getInstance()->fetchVertex(O_mesh);
		
		if(O_material)
		{
			if(O_material->getTexture())
			{
				glActiveTexture(GL_TEXTURE2);
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, O_material->getTexture()->getTextureId());
				glActiveTexture(GL_TEXTURE0);
				
				Xr_BufferManager::getInstance()->fetchTexCoord(O_mesh);
			}
		}
		
		Xr_BufferManager::getInstance()->fetchNormal(O_mesh);

		glDrawArrays(GL_TRIANGLES, 0, O_mesh->getFaceNumber() * 3);
		
		Xr_BufferManager::getInstance()->fetchNull();
	}
}

void Xr_Object::setPos(float x, float y, float z)
{
	O_pos[0] = x;
	O_pos[1] = y;
	O_pos[2] = z;
	
	if(O_box)
	{
		O_box->setPos(O_pos[0], O_pos[1], O_pos[2]);
	}
}

bool Xr_Object::isReady() const
{
	return O_isReady;
}

float* const Xr_Object::getPos() const
{
	return (float* const) O_pos;
}

Xr_Material* const Xr_Object::getMaterial() const
{
	return (Xr_Material* const) O_material;
}

Xr_AABB* const Xr_Object::getBox() const
{
	return (Xr_AABB* const) O_box;
}
