#include <GL/glew.h>
#include <SDL/SDL.h>

#include "render.h"

#include "OGL_ext.h"
#include "buffer.h"
#include "ressource.h"
#include "event.h"
#include "debug.h"

Xr_Renderer* Xr_Renderer::R_instance = 0;

Xr_Renderer::Xr_Renderer() : isInit(false), R_scene(0)
{
	
}

Xr_Renderer::~Xr_Renderer()
{
	Xr_EventManager::quitInstance();
	Xr_BufferManager::quitInstance();
	Xr_RessourceManager::quitInstance();
	
	if(isInit)
	{
		SDL_Quit();
	}
}

Xr_Renderer* Xr_Renderer::getInstance()
{
	if(R_instance)
	{
		return R_instance;
	}
	else
	{
		R_instance = new Xr_Renderer;
		return R_instance;
	}
}

bool Xr_Renderer::instanced()
{
	if(R_instance)
	{
		return true;
	}
	else
	{
		return false;
	}
}

void Xr_Renderer::quitInstance()
{
	if(R_instance)
	{
		delete R_instance;
		R_instance = 0;
	}
}

void Xr_Renderer::init(unsigned int width, unsigned int height, bool fullscreen, unsigned int framerate)
{
	R_fullscreen = fullscreen;
	R_frameRateLimite = framerate;
	R_windowWidth = width;
	R_imageWidth = width;
	R_windowHeight = height;
	R_imageHeight = height;

	SDL_Init(SDL_INIT_VIDEO);
	
	if(fullscreen)
	{
		SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_DOUBLEBUF | SDL_FULLSCREEN);
	}
	else
	{
		SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_DOUBLEBUF /*| SDL_RESIZABLE*/);
	}
	
	SDL_WM_SetCaption("X-Ray3D Render Window", NULL);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	Xr_InitExtSupport();
	Xr_CheckExt("GL_ARB_vertex_buffer_object");
	Xr_CheckExt("GL_ARB_shading_language_100");
	Xr_CheckExt("GL_ARB_shader_objects");
	Xr_CheckExt("GL_ARB_vertex_shader");
	Xr_CheckExt("GL_ARB_fragment_shader");
	
	isInit = true;
}

void Xr_Renderer::mainLoop()
{
	if(isInit)
	{
		if(R_scene)
		{
			Uint32 startTime, endTime, elapsedTime;
			while(!(Xr_EventManager::getInstance()->quitEvent()))
			{
				startTime = SDL_GetTicks();
			
				Xr_EventManager::getInstance()->updateDeviceState();
				Xr_EventManager::getInstance()->executeProfil();
			
				R_scene->draw();
			
				endTime = SDL_GetTicks();
				elapsedTime = endTime - startTime;
				
				if(elapsedTime < (1000 / R_frameRateLimite))
				{
					SDL_Delay((1000 / R_frameRateLimite) - elapsedTime);
					R_frameRate = R_frameRateLimite;
				}
				else
				{
					R_frameRate = 1000 / elapsedTime;
				}
			}
		}
		else
		{
			Xr_Log("Xr_Renderer : Pas de scène");
		}
	}
	else
	{
		Xr_Log("Xr_Renderer : Le renderer n'est pas initialisé");
	}
}

void Xr_Renderer::setVideo(unsigned int width, unsigned int height, bool fullscreen)
{
	R_fullscreen = fullscreen;
	R_windowWidth = width;
	R_windowHeight = height;

	if(fullscreen)
	{
		SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_DOUBLEBUF | SDL_FULLSCREEN);
	}
	else
	{
		SDL_SetVideoMode(width, height, 32, SDL_OPENGL | SDL_DOUBLEBUF /*| SDL_RESIZABLE*/);
	}
	SDL_Delay(1000);
}

void Xr_Renderer::setFrameRateLimite(unsigned int framerate)
{
	R_frameRateLimite = framerate;
}

void Xr_Renderer::setImageSize(unsigned int width, unsigned int height)
{
	R_imageWidth = width;
	R_imageHeight = height;
}

void Xr_Renderer::setScene(Xr_Scene *scene)
{
	if(scene)
	{
		R_scene = scene;
	}
	else
	{
		Xr_Log("Xr_Renderer : La scène est invalide");
	}
}

unsigned int Xr_Renderer::getWindowWidth() const
{
	return R_windowWidth;
}

unsigned int Xr_Renderer::getWindowHeight() const
{
	return R_windowHeight;
}

unsigned int Xr_Renderer::getImageWidth() const
{
	return R_imageWidth;
}

unsigned int Xr_Renderer::getImageHeight() const
{
	return R_imageHeight;
}

unsigned int Xr_Renderer::getFrameRate() const
{
	return R_frameRate;
}
