#include <GL/glew.h>
#include <SDL/SDL.h>

#include "scene.h"

#include "debug.h"
#include "ressource.h"
#include "render.h"

Xr_Scene::Xr_Scene() : modelview(new Xr_MATRIX_4X4), S_camera(0), basicProg(new Xr_Program), render1Prog(new Xr_Program)
{	
	basicProg->attachShader((Xr_Shader*) Xr_RessourceManager::getInstance()->get(xr_shader_type, "shader/basic.vert"));
	basicProg->attachShader((Xr_Shader*) Xr_RessourceManager::getInstance()->get(xr_shader_type, "shader/basic.frag"));
	basicProg->link();
	
	render1Prog->attachShader((Xr_Shader*) Xr_RessourceManager::getInstance()->get(xr_shader_type, "shader/render1.vert"));
	render1Prog->attachShader((Xr_Shader*) Xr_RessourceManager::getInstance()->get(xr_shader_type, "shader/render1.frag"));
	render1Prog->link();
}

Xr_Scene::~Xr_Scene()
{
	delete modelview;
	delete basicProg;
	delete render1Prog;
}

void Xr_Scene::addObject(Xr_Object *object)
{
	if(object)
	{
		if(object->isReady())
		{		
			S_objectList.push_back(object);
			Xr_Log("Xr_Scene : Un objet a été ajouté dans la scène");
		}
		else
		{
			Xr_Log("Xr_Scene : Impossible d'ajouter l'objet dans la scène");
			return;
		}
	}
	else
	{
		Xr_Log("Xr_Scene : L'objet est invalide");
	}
}

void Xr_Scene::deleteObject(Xr_Object *object)
{
	int i;
	for(i = 0; i < S_objectList.size(); i++)
	{
		if(S_objectList[i] == object)
		{
			S_objectList[i] = 0;
			return;
		}
	}
}

void Xr_Scene::addLight(Xr_Light *light)
{
	int i;
	if(light)
	{
		S_lightList.push_back(light);
		Xr_Log("Xr_Scene : Une source de lumière a été ajoutée dans la scène");
	}
	else
	{
		Xr_Log("Xr_Scene : La source de lumière est invalide");
	}
}

void Xr_Scene::deleteLight(Xr_Light *light)
{
	int i;
	for(i = 0; i < S_lightList.size(); i++)
	{
		if(S_lightList[i] == light)
		{
			S_lightList[i] = 0;
			return;
		}
	}
}

void Xr_Scene::attachCamera(Xr_Camera *camera)
{
	if(S_camera == 0)
	{
		if(camera)
		{
			Xr_Log("Xr_Scene : Une caméra a été ajoutée dans la scène");
			S_camera = camera;
		}
		else
		{
			Xr_Log("Xr_Scene : La caméra est invalide");
		}
	}
	else
	{
		Xr_Log("Xr_Scene : Une caméra est déjà attachée à la scène");
	}
	
}

void Xr_Scene::detachCamera()
{
	S_camera = 0;
}

void Xr_Scene::draw()
{
	int i, j;
	GLuint fb;
	bool lighting = false;

	if(!S_camera)
	{
		return;
	}
	
	glViewport(0, 0, Xr_Renderer::getInstance()->getImageWidth(), Xr_Renderer::getInstance()->getImageHeight());

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glClearColor(0.4, 0.4, 1, 0);

	glCullFace(GL_BACK);
			
	// ------------------ preparation des lumières ----------------------------
	/*glViewport(0, 0, DEPTH_MAP_SIZE, DEPTH_MAP_SIZE);
	glUseProgram(basicProg->getProgramId());
	for(i = 0; i < 15; i++)
	{
		if(S_light[i])
		{
			bool lighting = true;
			glGenFramebuffers(1, &fb);
			glBindFramebuffer(GL_FRAMEBUFFER, fb);
			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_TEXTURE_2D, S_light[i]->getLightDepthMap(), 0);

			glActiveTexture(GL_TEXTURE1);
			glEnable(GL_TEXTURE_2D);
			glBindTexture(GL_TEXTURE_2D, S_light[i]->getLightDepthMap());
			glActiveTexture(GL_TEXTURE0);

			glBindFramebuffer(GL_FRAMEBUFFER, fb);

			for(j = 0; j < 100; j++)
			{
				if(S_object[j])
				{	
					S_light[j]->getLightModelviewMatrix()->translate(S_object[j]->getPos()[0], S_object[j]->getPos()[1], S_object[j]->getPos()[2]);
					S_light[j]->getLightModelviewMatrix()->rotate(b, 0, 1, 0);

					glUniformMatrix4fv(glGetUniformLocation(basicProg->getProgramId(), "projection"), 1, GL_TRUE, S_light[i]->getLightProjectionMatrix()->getValues());
					glUniformMatrix4fv(glGetUniformLocation(basicProg->getProgramId(), "modelview"), 1, GL_TRUE, S_light[i]->getLightModelviewMatrix()->getValues());
					S_object[i]->draw();
				}
			}
			glDeleteFramebuffers(1, &fb);
		}
	}
	
	glUseProgram(0);
	glBindFramebuffer(GL_FRAMEBUFFER, 0);*/
				
	// ------------------------------------------------------------------
				
	// dessin du point de vue de la camera
				
	for(i = 0; i < S_objectList.size(); i++)
	{
		if(S_objectList[i])
		{
			modelview->loadIdentity();
			S_camera->configModelview(modelview);
			if(!(S_camera->getFrustumBox()->testAABB(S_objectList[i]->getBox())) || !S_objectList[i]->isReady())
			{
				//Xr_Log("Culling !");
				continue;
			}
			
			modelview->translate(S_objectList[i]->getPos()[0], S_objectList[i]->getPos()[1], S_objectList[i]->getPos()[2]);
			//modelview->rotate(b, 0, 1, 0);

			if(!lighting && !(S_objectList[i]->getMaterial()))
			{
				glUseProgram(basicProg->getProgramId());
				glUniformMatrix4fv(glGetUniformLocation(basicProg->getProgramId(), "projection"), 1, GL_TRUE, S_camera->getProjection()->getValues());
				glUniformMatrix4fv(glGetUniformLocation(basicProg->getProgramId(), "modelview"), 1, GL_TRUE, modelview->getValues());
				S_objectList[i]->draw();
				glUseProgram(0);
			}
			else if(!lighting && S_objectList[i]->getMaterial())
			{
				glUseProgram(render1Prog->getProgramId());
				glUniformMatrix4fv(glGetUniformLocation(render1Prog->getProgramId(), "projection"), 1, GL_TRUE, S_camera->getProjection()->getValues());
				glUniformMatrix4fv(glGetUniformLocation(render1Prog->getProgramId(), "modelview"), 1, GL_TRUE, modelview->getValues());
				glUniform1i(glGetUniformLocation(render1Prog->getProgramId(), "texture"), 2);
				S_objectList[i]->draw();
				glUseProgram(0);
			}
			else
			{
				continue;
			}
			/*glUseProgram(render1->getProgramId());
			glUniformMatrix4fv(glGetUniformLocation(render1->getProgramId(), "projection"), 1, GL_TRUE, projection->getValues());
			glUniformMatrix4fv(glGetUniformLocation(render1->getProgramId(), "modelview"), 1, GL_TRUE, modelview->getValues());

			glUniform1i(glGetUniformLocation(render1->getProgramId(), "lightDepthMap"), GL_TEXTURE1);
			glUniform1f(glGetUniformLocation(render1->getProgramId(), "ambient"), S_object[i]->getMaterial()->getAmbient());
				
			S_objectList[i]->draw();
				
			glUseProgram(0);*/
		}
	}
	SDL_GL_SwapBuffers();
}
