#include <cmath>

#include "vector.h"

Xr_Vector3D::Xr_Vector3D() : xCoord(0), yCoord(0), zCoord(0)
{
}

Xr_Vector3D::Xr_Vector3D(float x, float y, float z) : xCoord(x), yCoord(y), zCoord(z)
{
}

Xr_Vector3D::~Xr_Vector3D()
{
}

void Xr_Vector3D::setCoords(float x, float y, float z)
{
	xCoord = x;
	yCoord = y;
	zCoord = z;
}

void Xr_Vector3D::normalize()
{
	float length = sqrt((xCoord * xCoord) + (yCoord * yCoord) + (zCoord * zCoord));

	if(length != 0)
	{
		xCoord /= length;
		yCoord /= length;
		zCoord /= length;
	}
}

float Xr_Vector3D::dot(Xr_Vector3D const& vector1, Xr_Vector3D const& vector2)
{
	float dot = (vector1[0] * vector2[0]) + (vector1[1] * vector2[1]) + (vector1[2] * vector2[2]);
	
	return dot;
}

float Xr_Vector3D::getX() const
{
	return xCoord;
}

float Xr_Vector3D::getY() const
{
	return yCoord;
}

float Xr_Vector3D::getZ() const
{
	return zCoord;
}

Xr_Vector3D& Xr_Vector3D::operator=(Xr_Vector3D const& vector)
{
	xCoord = vector.getX();
	yCoord = vector.getY();
	zCoord = vector.getZ();
	
	return *this;
}

Xr_Vector3D& Xr_Vector3D::operator+=(Xr_Vector3D const& vector)
{
	xCoord += vector.getX();
	yCoord += vector.getY();
	zCoord += vector.getZ();
	
	return *this;
}

Xr_Vector3D& Xr_Vector3D::operator-=(Xr_Vector3D const& vector)
{
	xCoord -= vector.getX();
	yCoord -= vector.getY();
	zCoord -= vector.getZ();
	
	return *this;
}

Xr_Vector3D Xr_Vector3D::operator+(Xr_Vector3D const& vector) const
{
	float x = xCoord + vector.getX();
	float y = yCoord + vector.getY();
	float z = zCoord + vector.getZ();
	
	Xr_Vector3D vec(x, y, z);
	return vec;
}

Xr_Vector3D Xr_Vector3D::operator-(Xr_Vector3D const& vector) const
{
	float x = xCoord - vector.getX();
	float y = yCoord - vector.getY();
	float z = zCoord - vector.getZ();
	
	Xr_Vector3D vec(x, y, z);
	return vec;
}

Xr_Vector3D Xr_Vector3D::operator-() const
{
	float x = -xCoord;
	float y = -yCoord;
	float z = -zCoord;
	
	Xr_Vector3D vec(x, y, z);
	return vec;
}

Xr_Vector3D Xr_Vector3D::operator*(Xr_Vector3D const& vector) const
{
	float x = (yCoord * vector.getZ()) - (zCoord * vector.getY());
	float y = (zCoord * vector.getX()) - (xCoord * vector.getZ());
	float z = (xCoord * vector.getY()) - (yCoord * vector.getX());
	
	Xr_Vector3D vec(x, y, z);
	return vec;
}

Xr_Vector3D Xr_Vector3D::operator*(float n) const
{
	float x = xCoord * n;
	float y = yCoord * n;
	float z = zCoord * n;
	
	Xr_Vector3D vec(x, y, z);
	return vec;
}

float Xr_Vector3D::operator[](unsigned int n) const
{
	switch(n)
	{
		case 0:
			return xCoord;
		
		case 1:
			return yCoord;
		
		case 2:
			return zCoord;
			
		default:
			return 0;
	}
}
